# Copyright (C) 2024 Travis Abendshien (CyanVoxel).
# Licensed under the GPL-3.0 License.
# Created for TagStudio: https://github.com/CyanVoxel/TagStudio


import logging
import sys
import typing
from pathlib import Path
from PIL import Image, ImageQt
from PySide6.QtCore import Qt, QPropertyAnimation, QPoint, QEasingCurve
from PySide6.QtGui import QPixmap
from PySide6.QtWidgets import QWidget, QLabel, QVBoxLayout, QPushButton
from src.qt.widgets.clickable_label import ClickableLabel
from src.qt.helpers.color_overlay import gradient_overlay, theme_fg_overlay

# Only import for type checking/autocompletion, will not be imported at runtime.
if typing.TYPE_CHECKING:
    from src.qt.ts_qt import QtDriver

logging.basicConfig(format="%(message)s", level=logging.INFO)


class LandingWidget(QWidget):
    def __init__(self, driver: "QtDriver", pixel_ratio: float):
        super().__init__()
        self.driver: "QtDriver" = driver
        self.logo_label: ClickableLabel = ClickableLabel()
        self._pixel_ratio: float = pixel_ratio
        self._logo_width: int = int(480 * pixel_ratio)
        self._special_click_count: int = 0

        # Create layout --------------------------------------------------------
        self.landing_layout = QVBoxLayout()
        self.landing_layout.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.landing_layout.setSpacing(12)
        self.setLayout(self.landing_layout)

        # Create landing logo --------------------------------------------------
        # self.landing_logo_pixmap = QPixmap(":/images/tagstudio_logo_text_mono.png")
        self.logo_raw: Image.Image = Image.open(
            Path(__file__).parents[3]
            / "resources/qt/images/tagstudio_logo_text_mono.png"
        )
        self.landing_pixmap: QPixmap = QPixmap()
        self.update_logo_color()
        self.logo_label.clicked.connect(self._update_special_click)

        # Initialize landing logo animation ------------------------------------
        self.logo_pos_anim = QPropertyAnimation(self.logo_label, b"pos")
        self.logo_pos_anim.setEasingCurve(QEasingCurve.Type.OutCubic)
        self.logo_pos_anim.setDuration(1000)

        self.logo_special_anim = QPropertyAnimation(self.logo_label, b"pos")
        self.logo_special_anim.setEasingCurve(QEasingCurve.Type.OutCubic)
        self.logo_special_anim.setDuration(500)

        # Create "Open/Create Library" button ----------------------------------
        open_shortcut_text: str = ""
        if sys.platform == "darwin":
            open_shortcut_text = "(⌘+O)"
        else:
            open_shortcut_text = "(Ctrl+O)"
        self.open_button: QPushButton = QPushButton()
        self.open_button.setMinimumWidth(200)
        self.open_button.setText(f"Open/Create Library {open_shortcut_text}")
        self.open_button.clicked.connect(self.driver.open_library_from_dialog)

        # Create status label --------------------------------------------------
        self.status_label = QLabel()
        self.status_label.setMinimumWidth(200)
        self.status_label.setAlignment(Qt.AlignmentFlag.AlignCenter)
        self.status_label.setText("")

        # Initialize landing logo animation ------------------------------------
        self.status_pos_anim = QPropertyAnimation(self.status_label, b"pos")
        self.status_pos_anim.setEasingCurve(QEasingCurve.Type.OutCubic)
        self.status_pos_anim.setDuration(500)

        # Add widgets to layout ------------------------------------------------
        self.landing_layout.addWidget(self.logo_label)
        self.landing_layout.addWidget(
            self.open_button, alignment=Qt.AlignmentFlag.AlignCenter
        )
        self.landing_layout.addWidget(
            self.status_label, alignment=Qt.AlignmentFlag.AlignCenter
        )

    def update_logo_color(self, style: str = "mono"):
        """
        Update the color of the TagStudio logo.

        Args:
            style (str): = The style of the logo. Either "mono" or "gradient".
        """

        logo_im: Image.Image = None
        if style == "mono":
            logo_im = theme_fg_overlay(self.logo_raw)
        elif style == "gradient":
            gradient_colors: list[str] = ["#d27bf4", "#7992f5", "#63c6e3", "#63f5cf"]
            logo_im = gradient_overlay(self.logo_raw, gradient_colors)

        logo_final: Image.Image = Image.new(
            mode="RGBA", size=self.logo_raw.size, color="#00000000"
        )

        logo_final.paste(logo_im, (0, 0), mask=self.logo_raw)

        self.landing_pixmap = QPixmap.fromImage(ImageQt.ImageQt(logo_im))
        self.landing_pixmap.setDevicePixelRatio(self._pixel_ratio)
        self.landing_pixmap = self.landing_pixmap.scaledToWidth(
            self._logo_width, Qt.TransformationMode.SmoothTransformation
        )
        self.logo_label.setMaximumHeight(
            int(self.logo_raw.size[1] * (self.logo_raw.size[0] / self._logo_width))
        )
        self.logo_label.setMaximumWidth(self._logo_width)
        self.logo_label.setPixmap(self.landing_pixmap)

    def _update_special_click(self):
        """
        Increment the click count for the logo easter egg if it has not
        been triggered. If it reaches the click threshold, this triggers it
        and prevents it from triggering again.
        """
        if self._special_click_count >= 0:
            self._special_click_count += 1
            if self._special_click_count >= 10:
                self.update_logo_color("gradient")
                self.animate_logo_pop()
                self._special_click_count = -1

    def animate_logo_in(self):
        """Animate in the TagStudio logo."""
        # NOTE: Sometimes, mostly on startup without a library open, the
        # y position of logo_label is something like 10. I'm not sure what
        # the cause of this is, so I've just done this workaround to disable
        # the animation if the y position is too incorrect.
        if self.logo_label.y() > 50:
            self.logo_pos_anim.setStartValue(
                QPoint(self.logo_label.x(), self.logo_label.y() - 100)
            )
            self.logo_pos_anim.setEndValue(self.logo_label.pos())
            self.logo_pos_anim.start()

    def animate_logo_pop(self):
        """Special pop animation for the TagStudio logo."""
        self.logo_special_anim.setStartValue(self.logo_label.pos())
        self.logo_special_anim.setKeyValueAt(
            0.25, QPoint(self.logo_label.x() - 5, self.logo_label.y())
        )
        self.logo_special_anim.setKeyValueAt(
            0.5, QPoint(self.logo_label.x() + 5, self.logo_label.y() - 10)
        )
        self.logo_special_anim.setKeyValueAt(
            0.75, QPoint(self.logo_label.x() - 5, self.logo_label.y())
        )
        self.logo_special_anim.setEndValue(self.logo_label.pos())

        self.logo_special_anim.start()

    # def animate_status(self):
    #     # if self.status_label.y() > 50:
    #     logging.info(f"{self.status_label.pos()}")
    #     self.status_pos_anim.setStartValue(
    #         QPoint(self.status_label.x(), self.status_label.y() + 50)
    #     )
    #     self.status_pos_anim.setEndValue(self.status_label.pos())
    #     self.status_pos_anim.start()

    def set_status_label(self, text=str):
        """
        Set the text of the status label.

        Args:
            text (str): Text of the status to set.
        """
        # if text:
        #     self.animate_status()
        self.status_label.setText(text)



# -*- coding: utf-8 -*-

################################################################################
## Form generated from reading UI file 'home.ui'
##
## Created by: Qt User Interface Compiler version 6.6.3
##
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import (QCoreApplication, QDate, QDateTime, QLocale,
    QMetaObject, QObject, QPoint, QRect,
    QSize, QTime, QUrl, Qt)
from PySide6.QtGui import (QBrush, QColor, QConicalGradient, QCursor,
    QFont, QFontDatabase, QGradient, QIcon,
    QImage, QKeySequence, QLinearGradient, QPainter,
    QPalette, QPixmap, QRadialGradient, QTransform)
from PySide6.QtWidgets import (QApplication, QComboBox, QFrame, QGridLayout,
    QHBoxLayout, QLayout, QLineEdit, QMainWindow,
    QMenuBar, QPushButton, QScrollArea, QSizePolicy,
    QSpacerItem, QStatusBar, QWidget)

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        if not MainWindow.objectName():
            MainWindow.setObjectName(u"MainWindow")
        MainWindow.resize(1280, 720)
        self.centralwidget = QWidget(MainWindow)
        self.centralwidget.setObjectName(u"centralwidget")
        self.gridLayout = QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName(u"gridLayout")
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.horizontalSpacer = QSpacerItem(40, 20, QSizePolicy.Policy.Expanding, QSizePolicy.Policy.Minimum)

        self.horizontalLayout_3.addItem(self.horizontalSpacer)

        self.comboBox_2 = QComboBox(self.centralwidget)
        self.comboBox_2.addItem("")
        self.comboBox_2.addItem("")
        self.comboBox_2.setObjectName(u"comboBox_2")
        self.comboBox_2.setMinimumSize(QSize(165, 0))

        self.horizontalLayout_3.addWidget(self.comboBox_2)

        self.comboBox = QComboBox(self.centralwidget)
        self.comboBox.setObjectName(u"comboBox")
        sizePolicy = QSizePolicy(QSizePolicy.Policy.Minimum, QSizePolicy.Policy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.comboBox.sizePolicy().hasHeightForWidth())
        self.comboBox.setSizePolicy(sizePolicy)
        self.comboBox.setMinimumSize(QSize(128, 0))
        self.comboBox.setMaximumSize(QSize(256, 32))

        self.horizontalLayout_3.addWidget(self.comboBox)


        self.gridLayout.addLayout(self.horizontalLayout_3, 5, 0, 1, 1)

        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.scrollArea = QScrollArea(self.centralwidget)
        self.scrollArea.setObjectName(u"scrollArea")
        self.scrollArea.setFocusPolicy(Qt.WheelFocus)
        self.scrollArea.setFrameShape(QFrame.NoFrame)
        self.scrollArea.setFrameShadow(QFrame.Plain)
        self.scrollArea.setWidgetResizable(True)
        self.scrollAreaWidgetContents = QWidget()
        self.scrollAreaWidgetContents.setObjectName(u"scrollAreaWidgetContents")
        self.scrollAreaWidgetContents.setGeometry(QRect(0, 0, 1260, 585))
        self.gridLayout_2 = QGridLayout(self.scrollAreaWidgetContents)
        self.gridLayout_2.setSpacing(8)
        self.gridLayout_2.setObjectName(u"gridLayout_2")
        self.gridLayout_2.setContentsMargins(0, 0, 0, 8)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)

        self.horizontalLayout.addWidget(self.scrollArea)


        self.gridLayout.addLayout(self.horizontalLayout, 9, 0, 1, 1)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout_2.setSizeConstraint(QLayout.SetMinimumSize)
        self.backButton = QPushButton(self.centralwidget)
        self.backButton.setObjectName(u"backButton")
        self.backButton.setMinimumSize(QSize(0, 32))
        self.backButton.setMaximumSize(QSize(32, 16777215))
        font = QFont()
        font.setPointSize(14)
        font.setBold(True)
        self.backButton.setFont(font)

        self.horizontalLayout_2.addWidget(self.backButton)

        self.forwardButton = QPushButton(self.centralwidget)
        self.forwardButton.setObjectName(u"forwardButton")
        self.forwardButton.setMinimumSize(QSize(0, 32))
        self.forwardButton.setMaximumSize(QSize(32, 16777215))
        font1 = QFont()
        font1.setPointSize(14)
        font1.setBold(True)
        font1.setKerning(True)
        self.forwardButton.setFont(font1)

        self.horizontalLayout_2.addWidget(self.forwardButton)

        self.searchField = QLineEdit(self.centralwidget)
        self.searchField.setObjectName(u"searchField")
        self.searchField.setMinimumSize(QSize(0, 32))
        font2 = QFont()
        font2.setPointSize(11)
        font2.setBold(False)
        self.searchField.setFont(font2)

        self.horizontalLayout_2.addWidget(self.searchField)

        self.searchButton = QPushButton(self.centralwidget)
        self.searchButton.setObjectName(u"searchButton")
        self.searchButton.setMinimumSize(QSize(0, 32))
        self.searchButton.setFont(font2)

        self.horizontalLayout_2.addWidget(self.searchButton)


        self.gridLayout.addLayout(self.horizontalLayout_2, 3, 0, 1, 1)

        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QMenuBar(MainWindow)
        self.menubar.setObjectName(u"menubar")
        self.menubar.setGeometry(QRect(0, 0, 1280, 21))
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QStatusBar(MainWindow)
        self.statusbar.setObjectName(u"statusbar")
        sizePolicy1 = QSizePolicy(QSizePolicy.Policy.Preferred, QSizePolicy.Policy.Maximum)
        sizePolicy1.setHorizontalStretch(0)
        sizePolicy1.setVerticalStretch(0)
        sizePolicy1.setHeightForWidth(self.statusbar.sizePolicy().hasHeightForWidth())
        self.statusbar.setSizePolicy(sizePolicy1)
        MainWindow.setStatusBar(self.statusbar)

        self.retranslateUi(MainWindow)

        QMetaObject.connectSlotsByName(MainWindow)
    # setupUi

    def retranslateUi(self, MainWindow):
        MainWindow.setWindowTitle(QCoreApplication.translate("MainWindow", u"MainWindow", None))
        self.comboBox_2.setItemText(0, QCoreApplication.translate("MainWindow", u"And (includes all tags)", None))
        self.comboBox_2.setItemText(1, QCoreApplication.translate("MainWindow", u"Or (includes any tag)", None))

        self.comboBox.setCurrentText("")
        self.comboBox.setPlaceholderText(QCoreApplication.translate("MainWindow", u"Thumbnail Size", None))
        self.backButton.setText(QCoreApplication.translate("MainWindow", u"<", None))
        self.forwardButton.setText(QCoreApplication.translate("MainWindow", u">", None))
        self.searchField.setPlaceholderText(QCoreApplication.translate("MainWindow", u"Search Entries", None))
        self.searchButton.setText(QCoreApplication.translate("MainWindow", u"Search", None))
    # retranslateUi




# Copyright (C) 2024 Travis Abendshien (CyanVoxel).
# Licensed under the GPL-3.0 License.
# Created for TagStudio: https://github.com/CyanVoxel/TagStudio


def clean_folder_name(folder_name: str) -> str:
    cleaned_name = folder_name
    invalid_chars = '<>:"/\\|?*.'
    for char in invalid_chars:
        cleaned_name = cleaned_name.replace(char, "_")
    return cleaned_name



# Copyright (C) 2024 Travis Abendshien (CyanVoxel).
# Licensed under the GPL-3.0 License.
# Created for TagStudio: https://github.com/CyanVoxel/TagStudio


import logging
import math
import typing

from PySide6.QtCore import Signal, Qt
from PySide6.QtWidgets import QPushButton

from src.core.constants import TAG_FAVORITE, TAG_ARCHIVED
from src.core.library import Library, Tag
from src.qt.flowlayout import FlowLayout
from src.qt.widgets.fields import FieldWidget
from src.qt.widgets.tag import TagWidget
from src.qt.widgets.panel import PanelModal
from src.qt.modals.build_tag import BuildTagPanel
from src.qt.modals.tag_search import TagSearchPanel

# Only import for type checking/autocompletion, will not be imported at runtime.
if typing.TYPE_CHECKING:
    from src.qt.ts_qt import QtDriver


class TagBoxWidget(FieldWidget):
    updated = Signal()

    def __init__(
        self,
        item,
        title,
        field_index,
        library: Library,
        tags: list[int],
        driver: "QtDriver",
    ) -> None:
        super().__init__(title)
        # QObject.__init__(self)
        self.item = item
        self.lib = library
        self.driver = driver  # Used for creating tag click callbacks that search entries for that tag.
        self.field_index = field_index
        self.tags: list[int] = tags
        self.setObjectName("tagBox")
        self.base_layout = FlowLayout()
        self.base_layout.setGridEfficiency(False)
        self.base_layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(self.base_layout)

        self.add_button = QPushButton()
        self.add_button.setCursor(Qt.CursorShape.PointingHandCursor)
        self.add_button.setMinimumSize(23, 23)
        self.add_button.setMaximumSize(23, 23)
        self.add_button.setText("+")
        self.add_button.setStyleSheet(
            f"QPushButton{{"
            f"background: #1e1e1e;"
            f"color: #FFFFFF;"
            f"font-weight: bold;"
            f"border-color: #333333;"
            f"border-radius: 6px;"
            f"border-style:solid;"
            f"border-width:{math.ceil(1*self.devicePixelRatio())}px;"
            # f'padding-top: 1.5px;'
            # f'padding-right: 4px;'
            f"padding-bottom: 5px;"
            # f'padding-left: 4px;'
            f"font-size: 20px;"
            f"}}"
            f"QPushButton::hover"
            f"{{"
            f"border-color: #CCCCCC;"
            f"background: #555555;"
            f"}}"
        )
        tsp = TagSearchPanel(self.lib)
        tsp.tag_chosen.connect(lambda x: self.add_tag_callback(x))
        self.add_modal = PanelModal(tsp, title, "Add Tags")
        self.add_button.clicked.connect(
            lambda: (tsp.update_tags(), self.add_modal.show())  # type: ignore
        )

        self.set_tags(tags)
        # self.add_button.setHidden(True)

    def set_item(self, item):
        self.item = item

    def set_tags(self, tags: list[int]):
        logging.info(f"[TAG BOX WIDGET] SET TAGS: T:{tags} for E:{self.item.id}")
        is_recycled = False
        if self.base_layout.itemAt(0):
            # logging.info(type(self.base_layout.itemAt(0).widget()))
            while self.base_layout.itemAt(0) and self.base_layout.itemAt(1):
                # logging.info(f"I'm deleting { self.base_layout.itemAt(0).widget()}")
                self.base_layout.takeAt(0).widget().deleteLater()
            is_recycled = True
        for tag in tags:
            # TODO: Remove space from the special search here (tag_id:x) once that system is finalized.
            # tw = TagWidget(self.lib, self.lib.get_tag(tag), True, True,
            # 							on_remove_callback=lambda checked=False, t=tag: (self.lib.get_entry(self.item.id).remove_tag(self.lib, t, self.field_index), self.updated.emit()),
            # 							on_click_callback=lambda checked=False, q=f'tag_id: {tag}': (self.driver.main_window.searchField.setText(q), self.driver.filter_items(q)),
            # 							on_edit_callback=lambda checked=False, t=tag: (self.edit_tag(t))
            # 							)
            tw = TagWidget(self.lib, self.lib.get_tag(tag), True, True)
            tw.on_click.connect(
                lambda checked=False, q=f"tag_id: {tag}": (
                    self.driver.main_window.searchField.setText(q),
                    self.driver.filter_items(q),
                )
            )
            tw.on_remove.connect(lambda checked=False, t=tag: (self.remove_tag(t)))
            tw.on_edit.connect(lambda checked=False, t=tag: (self.edit_tag(t)))
            self.base_layout.addWidget(tw)
        self.tags = tags

        # Move or add the '+' button.
        if is_recycled:
            self.base_layout.addWidget(self.base_layout.takeAt(0).widget())
        else:
            self.base_layout.addWidget(self.add_button)

        # Handles an edge case where there are no more tags and the '+' button
        # doesn't move all the way to the left.
        if self.base_layout.itemAt(0) and not self.base_layout.itemAt(1):
            self.base_layout.update()

    def edit_tag(self, tag_id: int):
        btp = BuildTagPanel(self.lib, tag_id)
        # btp.on_edit.connect(lambda x: self.edit_tag_callback(x))
        self.edit_modal = PanelModal(
            btp,
            self.lib.get_tag(tag_id).display_name(self.lib),
            "Edit Tag",
            done_callback=(self.driver.preview_panel.update_widgets),
            has_save=True,
        )
        # self.edit_modal.widget.update_display_name.connect(lambda t: self.edit_modal.title_widget.setText(t))
        self.edit_modal.saved.connect(lambda: self.lib.update_tag(btp.build_tag()))
        # panel.tag_updated.connect(lambda tag: self.lib.update_tag(tag))
        self.edit_modal.show()

    def add_tag_callback(self, tag_id: int):
        # self.base_layout.addWidget(TagWidget(self.lib, self.lib.get_tag(tag), True))
        # self.tags.append(tag)
        logging.info(
            f"[TAG BOX WIDGET] ADD TAG CALLBACK: T:{tag_id} to E:{self.item.id}"
        )
        logging.info(f"[TAG BOX WIDGET] SELECTED T:{self.driver.selected}")
        id: int = list(self.field.keys())[0]  # type: ignore
        for x in self.driver.selected:
            self.driver.lib.get_entry(x[1]).add_tag(
                self.driver.lib, tag_id, field_id=id, field_index=-1
            )
            self.updated.emit()
        if tag_id in (TAG_FAVORITE, TAG_ARCHIVED):
            self.driver.update_badges()

        # if type((x[0]) == ThumbButton):
        # 	# TODO: Remove space from the special search here (tag_id:x) once that system is finalized.
        # logging.info(f'I want to add tag ID {tag_id} to entry {self.item.filename}')
        # self.updated.emit()
        # if tag_id not in self.tags:
        # 	self.tags.append(tag_id)
        # self.set_tags(self.tags)
        # elif type((x[0]) == ThumbButton):

    def edit_tag_callback(self, tag: Tag):
        self.lib.update_tag(tag)

    def remove_tag(self, tag_id: int):
        logging.info(f"[TAG BOX WIDGET] SELECTED T:{self.driver.selected}")
        id: int = list(self.field.keys())[0]  # type: ignore
        for x in self.driver.selected:
            index = self.driver.lib.get_field_index_in_entry(
                self.driver.lib.get_entry(x[1]), id
            )
            self.driver.lib.get_entry(x[1]).remove_tag(
                self.driver.lib, tag_id, field_index=index[0]
            )
            self.updated.emit()
        if tag_id in (TAG_FAVORITE, TAG_ARCHIVED):
            self.driver.update_badges()

    # def show_add_button(self, value:bool):
    # 	self.add_button.setHidden(not value)



# Copyright (C) 2024 Travis Abendshien (CyanVoxel).
# Licensed under the GPL-3.0 License.
# Created for TagStudio: https://github.com/CyanVoxel/TagStudio

from PySide6.QtWidgets import QPushButton


class QPushButtonWrapper(QPushButton):
    """
    This is a customized implementation of the PySide6 QPushButton that allows to suppress the warning that is triggered
    by disconnecting a signal that is not currently connected.
    """

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.is_connected = False



import enum


class SettingItems(str, enum.Enum):
    """List of setting item names."""

    START_LOAD_LAST = "start_load_last"
    LAST_LIBRARY = "last_library"
    LIBS_LIST = "libs_list"
    WINDOW_SHOW_LIBS = "window_show_libs"
    AUTOPLAY = "autoplay_videos"


class Theme(str, enum.Enum):
    COLOR_BG = "#65000000"
    COLOR_HOVER = "#65AAAAAA"
    COLOR_PRESSED = "#65EEEEEE"
    COLOR_DISABLED = "#65F39CAA"
    COLOR_DISABLED_BG = "#65440D12"


class SearchMode(int, enum.Enum):
    """Operational modes for item searching."""

    AND = 0
    OR = 1


class FieldID(int, enum.Enum):
    TITLE = 0
    AUTHOR = 1
    ARTIST = 2
    DESCRIPTION = 4
    NOTES = 5
    TAGS = 6
    CONTENT_TAGS = 7
    META_TAGS = 8
    DATE_PUBLISHED = 14
    SOURCE = 21



# Copyright (C) 2024 Travis Abendshien (CyanVoxel).
# Licensed under the GPL-3.0 License.
# Created for TagStudio: https://github.com/CyanVoxel/TagStudio


from PySide6.QtWidgets import QVBoxLayout, QPlainTextEdit

from src.qt.widgets.panel import PanelWidget


class EditTextBox(PanelWidget):
    def __init__(self, text):
        super().__init__()
        # self.setLayout()
        self.setMinimumSize(480, 480)
        self.root_layout = QVBoxLayout(self)
        self.root_layout.setContentsMargins(6, 0, 6, 0)
        self.text = text
        self.text_edit = QPlainTextEdit()
        self.text_edit.setPlainText(text)
        self.root_layout.addWidget(self.text_edit)

    def get_content(self) -> str:
        return self.text_edit.toPlainText()

    def reset(self):
        self.text_edit.setPlainText(self.text)



# Copyright (C) 2024 Travis Abendshien (CyanVoxel).
# Licensed under the GPL-3.0 License.
# Created for TagStudio: https://github.com/CyanVoxel/TagStudio

from PIL import Image, ImageEnhance, ImageChops


def four_corner_gradient_background(
    image: Image.Image, adj_size, mask, hl
) -> Image.Image:
    if image.size != (adj_size, adj_size):
        # Old 1 color method.
        # bg_col = image.copy().resize((1, 1)).getpixel((0,0))
        # bg = Image.new(mode='RGB',size=(adj_size,adj_size),color=bg_col)
        # bg.thumbnail((1, 1))
        # bg = bg.resize((adj_size,adj_size), resample=Image.Resampling.NEAREST)

        # Small gradient background. Looks decent, and is only a one-liner.
        # bg = image.copy().resize((2, 2), resample=Image.Resampling.BILINEAR).resize((adj_size,adj_size),resample=Image.Resampling.BILINEAR)

        # Four-Corner Gradient Background.
        # Not exactly a one-liner, but it's (subjectively) really cool.
        tl = image.getpixel((0, 0))
        tr = image.getpixel(((image.size[0] - 1), 0))
        bl = image.getpixel((0, (image.size[1] - 1)))
        br = image.getpixel(((image.size[0] - 1), (image.size[1] - 1)))
        bg = Image.new(mode="RGB", size=(2, 2))
        bg.paste(tl, (0, 0, 2, 2))
        bg.paste(tr, (1, 0, 2, 2))
        bg.paste(bl, (0, 1, 2, 2))
        bg.paste(br, (1, 1, 2, 2))
        bg = bg.resize((adj_size, adj_size), resample=Image.Resampling.BICUBIC)

        bg.paste(
            image,
            box=(
                (adj_size - image.size[0]) // 2,
                (adj_size - image.size[1]) // 2,
            ),
        )

        bg.putalpha(mask)
        final = bg

    else:
        image.putalpha(mask)
        final = image

    hl_soft = hl.copy()
    hl_soft.putalpha(ImageEnhance.Brightness(hl.getchannel(3)).enhance(0.5))
    final.paste(ImageChops.soft_light(final, hl_soft), mask=hl_soft.getchannel(3))
    return final


def linear_gradient(
    size=tuple[int, int],
    colors=list[str],
    interpolation: Image.Resampling = Image.Resampling.BICUBIC,
) -> Image.Image:
    seed: Image.Image = Image.new(mode="RGBA", size=(len(colors), 1), color="#000000")
    for i, color in enumerate(colors):
        c_im: Image.Image = Image.new(mode="RGBA", size=(1, 1), color=color)
        seed.paste(c_im, (i, 0))
    gradient: Image.Image = seed.resize(size, resample=interpolation)
    return gradient



# Copyright (C) 2024 Travis Abendshien (CyanVoxel).
# Licensed under the GPL-3.0 License.
# Created for TagStudio: https://github.com/CyanVoxel/TagStudio


import logging
import math

from PySide6.QtCore import Signal, Qt, QSize
from PySide6.QtWidgets import (
    QWidget,
    QVBoxLayout,
    QHBoxLayout,
    QPushButton,
    QLineEdit,
    QScrollArea,
    QFrame,
)

from src.core.library import Library
from src.core.palette import ColorType, get_tag_color
from src.qt.widgets.panel import PanelWidget
from src.qt.widgets.tag import TagWidget


ERROR = f"[ERROR]"
WARNING = f"[WARNING]"
INFO = f"[INFO]"

logging.basicConfig(format="%(message)s", level=logging.INFO)


class TagSearchPanel(PanelWidget):
    tag_chosen = Signal(int)

    def __init__(self, library):
        super().__init__()
        self.lib: Library = library
        # self.callback = callback
        self.first_tag_id = None
        self.tag_limit = 100
        # self.selected_tag: int = 0
        self.setMinimumSize(300, 400)
        self.root_layout = QVBoxLayout(self)
        self.root_layout.setContentsMargins(6, 0, 6, 0)

        self.search_field = QLineEdit()
        self.search_field.setObjectName("searchField")
        self.search_field.setMinimumSize(QSize(0, 32))
        self.search_field.setPlaceholderText("Search Tags")
        self.search_field.textEdited.connect(
            lambda x=self.search_field.text(): self.update_tags(x)
        )
        self.search_field.returnPressed.connect(
            lambda checked=False: self.on_return(self.search_field.text())
        )

        # self.content_container = QWidget()
        # self.content_layout = QHBoxLayout(self.content_container)

        self.scroll_contents = QWidget()
        self.scroll_layout = QVBoxLayout(self.scroll_contents)
        self.scroll_layout.setContentsMargins(6, 0, 6, 0)
        self.scroll_layout.setAlignment(Qt.AlignmentFlag.AlignTop)

        self.scroll_area = QScrollArea()
        # self.scroll_area.setStyleSheet('background: #000000;')
        self.scroll_area.setVerticalScrollBarPolicy(
            Qt.ScrollBarPolicy.ScrollBarAlwaysOn
        )
        # self.scroll_area.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
        self.scroll_area.setWidgetResizable(True)
        self.scroll_area.setFrameShadow(QFrame.Shadow.Plain)
        self.scroll_area.setFrameShape(QFrame.Shape.NoFrame)
        # sa.setMaximumWidth(self.preview_size[0])
        self.scroll_area.setWidget(self.scroll_contents)

        # self.add_button = QPushButton()
        # self.root_layout.addWidget(self.add_button)
        # self.add_button.setText('Add Tag')
        # # self.done_button.clicked.connect(lambda checked=False, x=1101: (callback(x), self.hide()))
        # self.add_button.clicked.connect(lambda checked=False, x=1101: callback(x))
        # # self.setLayout(self.root_layout)

        self.root_layout.addWidget(self.search_field)
        self.root_layout.addWidget(self.scroll_area)
        self.update_tags("")

    # def reset(self):
    # 	self.search_field.setText('')
    # 	self.update_tags('')
    # 	self.search_field.setFocus()

    def on_return(self, text: str):
        if text and self.first_tag_id is not None:
            # callback(self.first_tag_id)
            self.tag_chosen.emit(self.first_tag_id)
            self.search_field.setText("")
            self.update_tags()
        else:
            self.search_field.setFocus()
            self.parentWidget().hide()

    def update_tags(self, query: str = ""):
        # for c in self.scroll_layout.children():
        # 	c.widget().deleteLater()
        while self.scroll_layout.count():
            # logging.info(f"I'm deleting { self.scroll_layout.itemAt(0).widget()}")
            self.scroll_layout.takeAt(0).widget().deleteLater()

        found_tags = self.lib.search_tags(query, include_cluster=True)[: self.tag_limit]
        self.first_tag_id = found_tags[0] if found_tags else None

        for tag_id in found_tags:
            c = QWidget()
            l = QHBoxLayout(c)
            l.setContentsMargins(0, 0, 0, 0)
            l.setSpacing(3)
            tw = TagWidget(self.lib, self.lib.get_tag(tag_id), False, False)
            ab = QPushButton()
            ab.setMinimumSize(23, 23)
            ab.setMaximumSize(23, 23)
            ab.setText("+")
            ab.setStyleSheet(
                f"QPushButton{{"
                f"background: {get_tag_color(ColorType.PRIMARY, self.lib.get_tag(tag_id).color)};"
                # f'background-color: qlineargradient(x1:0, y1:0, x2:0, y2:1, stop:0 {get_tag_color(ColorType.PRIMARY, tag.color)}, stop:1.0 {get_tag_color(ColorType.BORDER, tag.color)});'
                # f"border-color:{get_tag_color(ColorType.PRIMARY, tag.color)};"
                f"color: {get_tag_color(ColorType.TEXT, self.lib.get_tag(tag_id).color)};"
                f"font-weight: 600;"
                f"border-color:{get_tag_color(ColorType.BORDER, self.lib.get_tag(tag_id).color)};"
                f"border-radius: 6px;"
                f"border-style:solid;"
                f"border-width: {math.ceil(1*self.devicePixelRatio())}px;"
                # f'padding-top: 1.5px;'
                # f'padding-right: 4px;'
                f"padding-bottom: 5px;"
                # f'padding-left: 4px;'
                f"font-size: 20px;"
                f"}}"
                f"QPushButton::hover"
                f"{{"
                f"border-color:{get_tag_color(ColorType.LIGHT_ACCENT, self.lib.get_tag(tag_id).color)};"
                f"color: {get_tag_color(ColorType.DARK_ACCENT, self.lib.get_tag(tag_id).color)};"
                f"background: {get_tag_color(ColorType.LIGHT_ACCENT, self.lib.get_tag(tag_id).color)};"
                f"}}"
            )

            ab.clicked.connect(lambda checked=False, x=tag_id: self.tag_chosen.emit(x))

            l.addWidget(tw)
            l.addWidget(ab)
            self.scroll_layout.addWidget(c)

        self.search_field.setFocus()

    # def enterEvent(self, event: QEnterEvent) -> None:
    # 	self.search_field.setFocus()
    # 	return super().enterEvent(event)
    # 	self.focusOutEvent



# Copyright (C) 2024 Travis Abendshien (CyanVoxel).
# Licensed under the GPL-3.0 License.
# Created for TagStudio: https://github.com/CyanVoxel/TagStudio

from enum import Enum


class ColorType(int, Enum):
    PRIMARY = 0
    TEXT = 1
    BORDER = 2
    LIGHT_ACCENT = 3
    DARK_ACCENT = 4


_TAG_COLORS = {
    "": {
        ColorType.PRIMARY: "#1e1e1e",
        ColorType.TEXT: ColorType.LIGHT_ACCENT,
        ColorType.BORDER: "#333333",
        ColorType.LIGHT_ACCENT: "#FFFFFF",
        ColorType.DARK_ACCENT: "#222222",
    },
    "black": {
        ColorType.PRIMARY: "#111018",
        ColorType.TEXT: ColorType.LIGHT_ACCENT,
        ColorType.BORDER: "#18171e",
        ColorType.LIGHT_ACCENT: "#b7b6be",
        ColorType.DARK_ACCENT: "#03020a",
    },
    "dark gray": {
        ColorType.PRIMARY: "#24232a",
        ColorType.TEXT: ColorType.LIGHT_ACCENT,
        ColorType.BORDER: "#2a2930",
        ColorType.LIGHT_ACCENT: "#bdbcc4",
        ColorType.DARK_ACCENT: "#07060e",
    },
    "gray": {
        ColorType.PRIMARY: "#53525a",
        ColorType.TEXT: ColorType.LIGHT_ACCENT,
        ColorType.BORDER: "#5b5a62",
        ColorType.LIGHT_ACCENT: "#cbcad2",
        ColorType.DARK_ACCENT: "#191820",
    },
    "light gray": {
        ColorType.PRIMARY: "#aaa9b0",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#b6b4bc",
        ColorType.LIGHT_ACCENT: "#cbcad2",
        ColorType.DARK_ACCENT: "#191820",
    },
    "white": {
        ColorType.PRIMARY: "#f2f1f8",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#fefeff",
        ColorType.LIGHT_ACCENT: "#ffffff",
        ColorType.DARK_ACCENT: "#302f36",
    },
    "light pink": {
        ColorType.PRIMARY: "#ff99c4",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#ffaad0",
        ColorType.LIGHT_ACCENT: "#ffcbe7",
        ColorType.DARK_ACCENT: "#6c2e3b",
    },
    "pink": {
        ColorType.PRIMARY: "#ff99c4",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#ffaad0",
        ColorType.LIGHT_ACCENT: "#ffcbe7",
        ColorType.DARK_ACCENT: "#6c2e3b",
    },
    "magenta": {
        ColorType.PRIMARY: "#f6466f",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#f7587f",
        ColorType.LIGHT_ACCENT: "#fba4bf",
        ColorType.DARK_ACCENT: "#61152f",
    },
    "red": {
        ColorType.PRIMARY: "#e22c3c",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#b21f2d",
        # ColorType.BORDER: '#e54252',
        ColorType.LIGHT_ACCENT: "#f39caa",
        ColorType.DARK_ACCENT: "#440d12",
    },
    "red orange": {
        ColorType.PRIMARY: "#e83726",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#ea4b3b",
        ColorType.LIGHT_ACCENT: "#f5a59d",
        ColorType.DARK_ACCENT: "#61120b",
    },
    "salmon": {
        ColorType.PRIMARY: "#f65848",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#f76c5f",
        ColorType.LIGHT_ACCENT: "#fcadaa",
        ColorType.DARK_ACCENT: "#6f1b16",
    },
    "orange": {
        ColorType.PRIMARY: "#ed6022",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#ef7038",
        ColorType.LIGHT_ACCENT: "#f7b79b",
        ColorType.DARK_ACCENT: "#551e0a",
    },
    "yellow orange": {
        ColorType.PRIMARY: "#fa9a2c",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#fba94b",
        ColorType.LIGHT_ACCENT: "#fdd7ab",
        ColorType.DARK_ACCENT: "#66330d",
    },
    "yellow": {
        ColorType.PRIMARY: "#ffd63d",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        #    ColorType.BORDER: '#ffe071',
        ColorType.BORDER: "#e8af31",
        ColorType.LIGHT_ACCENT: "#fff3c4",
        ColorType.DARK_ACCENT: "#754312",
    },
    "mint": {
        ColorType.PRIMARY: "#4aed90",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#79f2b1",
        ColorType.LIGHT_ACCENT: "#c8fbe9",
        ColorType.DARK_ACCENT: "#164f3e",
    },
    "lime": {
        ColorType.PRIMARY: "#92e649",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#b2ed72",
        ColorType.LIGHT_ACCENT: "#e9f9b7",
        ColorType.DARK_ACCENT: "#405516",
    },
    "light green": {
        ColorType.PRIMARY: "#85ec76",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#a3f198",
        ColorType.LIGHT_ACCENT: "#e7fbe4",
        ColorType.DARK_ACCENT: "#2b5524",
    },
    "green": {
        ColorType.PRIMARY: "#28bb48",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#43c568",
        ColorType.LIGHT_ACCENT: "#93e2c8",
        ColorType.DARK_ACCENT: "#0d3828",
    },
    "teal": {
        ColorType.PRIMARY: "#1ad9b2",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#4de3c7",
        ColorType.LIGHT_ACCENT: "#a0f3e8",
        ColorType.DARK_ACCENT: "#08424b",
    },
    "cyan": {
        ColorType.PRIMARY: "#49e4d5",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#76ebdf",
        ColorType.LIGHT_ACCENT: "#bff5f0",
        ColorType.DARK_ACCENT: "#0f4246",
    },
    "light blue": {
        ColorType.PRIMARY: "#55bbf6",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#70c6f7",
        ColorType.LIGHT_ACCENT: "#bbe4fb",
        ColorType.DARK_ACCENT: "#122541",
    },
    "blue": {
        ColorType.PRIMARY: "#3b87f0",
        ColorType.TEXT: ColorType.LIGHT_ACCENT,
        ColorType.BORDER: "#4e95f2",
        ColorType.LIGHT_ACCENT: "#aedbfa",
        ColorType.DARK_ACCENT: "#122948",
    },
    "blue violet": {
        ColorType.PRIMARY: "#5948f2",
        ColorType.TEXT: ColorType.LIGHT_ACCENT,
        ColorType.BORDER: "#6258f3",
        ColorType.LIGHT_ACCENT: "#9cb8fb",
        ColorType.DARK_ACCENT: "#1b1649",
    },
    "violet": {
        ColorType.PRIMARY: "#874ff5",
        ColorType.TEXT: ColorType.LIGHT_ACCENT,
        ColorType.BORDER: "#9360f6",
        ColorType.LIGHT_ACCENT: "#c9b0fa",
        ColorType.DARK_ACCENT: "#3a1860",
    },
    "purple": {
        ColorType.PRIMARY: "#bb4ff0",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#c364f2",
        ColorType.LIGHT_ACCENT: "#dda7f7",
        ColorType.DARK_ACCENT: "#531862",
    },
    "peach": {
        ColorType.PRIMARY: "#f1c69c",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#f4d4b4",
        ColorType.LIGHT_ACCENT: "#fbeee1",
        ColorType.DARK_ACCENT: "#613f2f",
    },
    "brown": {
        ColorType.PRIMARY: "#823216",
        ColorType.TEXT: ColorType.LIGHT_ACCENT,
        ColorType.BORDER: "#8a3e22",
        ColorType.LIGHT_ACCENT: "#cd9d83",
        ColorType.DARK_ACCENT: "#3a1804",
    },
    "lavender": {
        ColorType.PRIMARY: "#ad8eef",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#b99ef2",
        ColorType.LIGHT_ACCENT: "#d5c7fa",
        ColorType.DARK_ACCENT: "#492b65",
    },
    "blonde": {
        ColorType.PRIMARY: "#efc664",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#f3d387",
        ColorType.LIGHT_ACCENT: "#faebc6",
        ColorType.DARK_ACCENT: "#6d461e",
    },
    "auburn": {
        ColorType.PRIMARY: "#a13220",
        ColorType.TEXT: ColorType.LIGHT_ACCENT,
        ColorType.BORDER: "#aa402f",
        ColorType.LIGHT_ACCENT: "#d98a7f",
        ColorType.DARK_ACCENT: "#3d100a",
    },
    "light brown": {
        ColorType.PRIMARY: "#be5b2d",
        ColorType.TEXT: ColorType.DARK_ACCENT,
        ColorType.BORDER: "#c4693d",
        ColorType.LIGHT_ACCENT: "#e5b38c",
        ColorType.DARK_ACCENT: "#4c290e",
    },
    "dark brown": {
        ColorType.PRIMARY: "#4c2315",
        ColorType.TEXT: ColorType.LIGHT_ACCENT,
        ColorType.BORDER: "#542a1c",
        ColorType.LIGHT_ACCENT: "#b78171",
        ColorType.DARK_ACCENT: "#211006",
    },
    "cool gray": {
        ColorType.PRIMARY: "#515768",
        ColorType.TEXT: ColorType.LIGHT_ACCENT,
        ColorType.BORDER: "#5b6174",
        ColorType.LIGHT_ACCENT: "#9ea1c3",
        ColorType.DARK_ACCENT: "#181a37",
    },
    "warm gray": {
        ColorType.PRIMARY: "#625550",
        ColorType.TEXT: ColorType.LIGHT_ACCENT,
        ColorType.BORDER: "#6c5e57",
        ColorType.LIGHT_ACCENT: "#c0a392",
        ColorType.DARK_ACCENT: "#371d18",
    },
    "olive": {
        ColorType.PRIMARY: "#4c652e",
        ColorType.TEXT: ColorType.LIGHT_ACCENT,
        ColorType.BORDER: "#586f36",
        ColorType.LIGHT_ACCENT: "#b4c17a",
        ColorType.DARK_ACCENT: "#23300e",
    },
    "berry": {
        ColorType.PRIMARY: "#9f2aa7",
        ColorType.TEXT: ColorType.LIGHT_ACCENT,
        ColorType.BORDER: "#aa43b4",
        ColorType.LIGHT_ACCENT: "#cc8fdc",
        ColorType.DARK_ACCENT: "#41114a",
    },
}


def get_tag_color(type, color):
    color = color.lower()
    try:
        if type == ColorType.TEXT:
            return get_tag_color(_TAG_COLORS[color][type], color)
        else:
            return _TAG_COLORS[color][type]
    except KeyError:
        return "#FF00FF"
